/*
 *  $Id: gatherv.c,v 1.5 1993/11/24 00:49:13 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: gatherv.c,v 1.5 1993/11/24 00:49:13 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Gatherv - 

@*/
int MPI_Gatherv ( sendbuf, sendcnt, sendtype, 
		  recvbuf, displs, recvcnts, recvtype, 
		  root, comm )
void             *sendbuf;
int               sendcnt;
MPI_Datatype      sendtype;
void             *recvbuf;
int              *displs;
int              *recvcnts;
MPI_Datatype      recvtype;
int               root;
MPI_Comm          comm;
{
  MPI_Status status;
  int        size, rank;
  int        errno = MPI_SUCCESS;

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0) ||
       (root            <  0)           || 
       (comm->comm_type == MPIR_INTER) ) 
    return (errno); 
  
  /* Is root within the communicator? */
  MPI_Comm_size ( comm, &size );
  if ( root >= size )
    return (errno);

  /* Get my rank and switch communicators to the hidden collective */
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* If rank == root, then I recv lots, otherwise I send */
  if ( rank == root ) {
    int i, extent;

    MPI_Send(sendbuf, sendcnt, sendtype, root, MPIR_GATHERV_TAG, comm);

    MPI_Type_extent(recvtype, &extent);
    for ( i=0; i<size; i++ ) {
      MPI_Recv( (void *)((int)recvbuf+displs[i]*extent), recvcnts[i], recvtype, i,
		 MPIR_GATHERV_TAG, comm, &status );
    }
  }
  else 
    MPI_Send( sendbuf, sendcnt, sendtype, root, MPIR_GATHERV_TAG, comm );

  return (errno);
}

