/*
 *  $Id: make_dims.c,v 1.2 1993/11/12 12:09:55 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Make_dims - helps determine "squarest" possible processor grid
                based on the number of nodes and dimensions used

algorithm:
This function uses a heuristic to determine good numbers, but it
does not comply with the draft since the numbers might not be
the "squarest" possible.

@*/
int MPI_Make_dims ( nnodes, ndims, dims )
int  nnodes;
int  ndims;
int *dims;
{
  int i; 
  int total       = 1;
  int residue     = nnodes;
  int current_dim = 0;
  int num_fixed   = 0;
  int last_factor = 2;

  /* Check for valid arguments */
  if ( (nnodes < 1) ||
       (ndims < 1) ||
       (dims == (int *)0) )
    return (MPI_SUCCESS);

  /* Set up dimension list for processing, set 0's to 1, >0's to ->0's */
  for ( i=0; i<ndims; i++) {
    if ( dims[i] < 1 )
      dims[i] = 1;
    else {
      total *= dims[i];
      residue = residue / dims[i];
      dims[i] = -dims[i];
      num_fixed++;
    }
  }
    
  /* Is the product of ">0 dims" a factor of nnodes? */
  if ( (nnodes % total) != 0 )
    return (MPI_SUCCESS) ; /* error condition */

  /* Were all dimensions fixed before entering? */
  if ( num_fixed == ndims )
    if ( total == nnodes )
      return (MPI_SUCCESS);
    else
      return (MPI_SUCCESS); /* error condition */

  /* Now fill out the rest of the dimensions */
  while ( residue > 1 ) {
    if (dims[current_dim] > 0) 
      for ( i=last_factor; i<= residue; i++ )
	if ( (residue % i) == 0 ) {
	  last_factor = i;
	  residue /= i;
	  dims[current_dim] *= i;
	  break;
	}
    current_dim = (current_dim + 1) % ndims;
  }

  /* Set the preset dims back to their original state */
  for ( i=0; i<ndims; i++ )
    if (dims[i] < 0)
      dims[i] = -dims[i];

  return (MPI_SUCCESS);
}
